#  Copyright (c) 2019 Jeorme Douay <jerome@far-out.biz>
#  MIT Liecnce, see the licence file for more information.

import numpy
from scipy.interpolate import interp1d

class Engine(object):
    '''
    Engine class.
    The engine class contains the torque / power curve for the defined engine. The curve is defined at class creation by overwriting the n and t internal list.
    The power is automatically calculated from the engine speed ( n ) and the engine torque (t). Using the supplied points in the lists ( curve), a interpolation function is created allowing to return the interpolated torque or power for any engine speed between the supplied min and max.
    '''
    def __init__(self):
        '''
        By default, a curve is created with engine speed vs torque to build up the engine torque curve. The curve is interpolated between the defined points
        '''
        super().__init__()
        self.n=[600,1000,2000,4000,6000]
        self.t=[100,500,1000,800,100]
        #self.p = [self.n[i] * self.t[i] / 60 * 2 * numpy.pi for i in range(len(self.n)-1)]
        self.torque= interp1d(self.n,self.t)
        #self.power = interp1d(self.n, self.p)

    def Torque(self,speed,limit=100):
        '''
        Return the torque at an engine speed with a max percentage torque ( 100% is full torque)
        :param speed: engine speed in rpm
        :param limit: maximum percent torque from curve
        :return: engine torque in Nm
        '''
        return [min(x,self.MaxTorque()*limit/100) for x in self.torque(speed)]

    def Power(self,speed):
        '''
        Return the Power at a given engine speed and a percentage torque
        :param speed: engine speed in rpm
        :param limit: max percentage torque
        :return: engie power in watts
        '''
        return self.Torque(speed)*speed/60*2*3.14

    def MaxTorque(self):
        '''
        Get the maximum torque for the engine torque curve
        :return: max torque in Nm
        '''
        return max(self.t)
