#  Copyright (c) 2019 Jeorme Douay <jerome@far-out.biz>
#  MIT Liecnce, see the licence file for more information.

import glob, numpy, pandas, time, os

from pyPowerTrain.engine import Engine
from pyPowerTrain.gearbox import Gearbox
from pyPowerTrain.resistance import Resistance
from pyPowerTrain.measurement import Measurement

class Vehicle(object):
    '''
    Vehicle class is the main class of the pyPowerTrain module
    '''
    conf = {
        'name': 'vehicle',
        'gearbox': Gearbox(),
        'engine': Engine(),
        'resistance': Resistance(),
        'raxle': 2.5,
        'dynradius': 0.503,
        'measurements': 'path/to/dir/',
        'data': 'path/to/data/dir/',
        'project': 'project',
        'cx':0,
        'fr':0,
        'ro': 0
    }

    def __init__(self):
        super().__init__()
        self.conf['resistance']=Resistance(self.conf['cx'],self.conf['fr'],self.conf['ro'])

    def Ratio(self,gear):
        return self.gearbox.Y(gear)

    def Torque(self,n,pedal=100):
        return self.torque.Z(pedal,n)

    def DrivingResistanceMap(self,n_min=600, n_max=4000, gear=1):
        data=pandas.DataFrame(numpy.array(range(n_min,n_max+1,1)),columns=['n'])
        data['ratio'] = self.gearbox.Y(gear) * self.raxle
        data['v'] = self.EngineToSpeed(data['n'], data['ratio'])
        data = data.loc[data['v']<=50]
        data['Fr'] = self.resistance.Y(data['v'])
        data['Tr']= self.ForceToTorque(data['Fr'],data['ratio'])
        data['Tr_p'] = data['Tr']/self.torque.Z(100,1400)*100
        return data

    def EngineToSpeed(self,engine,ratio):
        res=engine/60/ratio*self.conf['dynradius']*2*numpy.pi
        return res.astype(float)

    def SpeedToEngine(self,speed,ratio):
        res=speed/(2*numpy.pi*self.conf['dynradius'])*ratio*60
        return res.astype(float)

    def TorqueToForce(self, t, ratio):
        return t*ratio*self.conf['dynradius']

    def ForceToTorque(self, f, ratio):
        return f*self.conf['dynradius']/ratio

    def Fm(self,mass,alpha):
        res=mass* 9.81 * numpy.cos(numpy.pi/2-numpy.arctan(alpha/100))
        return res.astype(float)

    def FMtoAlpha(self,fm,m):
        return numpy.arctan(numpy.pi/2-numpy.arccos(fm/m/9.81))*100

    def Ft(self,torque,ratio):
        res=torque*ratio*self.raxle * self.conf['dynradius']
        return res.astype(float)

    def FResist(self,speed,ratio=1,mass=32):
        f=self.fr.loc[self.fr['ratio']==ratio]
        #f = f.loc[f['mass'] == mass]
        return self.FrEquation(speed,f['a'].iloc[0],f['b'].iloc[0],f['c'].iloc[0])

    def Gamma(self, ft, fr, ftm):
        """
        Return the Douay'sche normalized force factor calculated from traction force and resistance force
        :param ft: traction force in N
        :param fr: resistance force in N
        :param ftm: maximum traction force in N
        :return: normalized factor
        """
        return (ft - fr) / ftm

    def TimeOnGear(self, n_min=600, n_max=2000, gear=1, mass=9000, grade=0, limit=100, offset=0):
        data=pandas.DataFrame(numpy.array(range(n_min,n_max+1,1)),columns=['n'])
        tmax=self.conf['engine'].MaxTorque()
        data['t']=self.conf['engine'].Torque(data['n'],limit)
        data['tm'] = self.conf['engine'].Torque(data['n'], 100)
        data['P']=self.conf['engine'].Power(data['n'],limit)
        data['Pm'] = self.conf['engine'].Power(data['n'], 100)
        data['ratio'] = self.conf['gearbox'].Ratio(gear) * self.raxle
        data['v'] = self.EngineToSpeed(data['n'], data['ratio'])
        data['Fr'] = self.conf['resistance'].Resistance(data['v'],mass)
        data['Fr']=data['Fr']+self.Fm(mass,grade)+offset
        data['acc']=(data['t']*data['ratio']/self.conf['dynradius']-data['Fr'])/mass
        data['tr']=self.ForceToTorque(data['Fr'],data['ratio'])
        data['te']=data['t']-data['tr']
        data['Pr'] = data['tr'] * data['n'] / 60 * 2 * numpy.pi
        data['Pe'] = data['P']-data['Pr']
        data['dn']=data['acc']/2/numpy.pi/self.conf['dynradius']*data['ratio']*60
        data['dt'] = 1 / data['dn']
        data['dt'].loc[data['dt']<0]=0
        data['time'] = data['dt'].cumsum()
        data['rt']=(data['t']-data['tr'])/data['tm']
        data['gear']=gear
        data['mass']=mass
        data['grade']=grade
        data['limit']=limit
        data['offset']=offset
        return data

